from uuid import uuid4
from collections import defaultdict
from pyproj import Proj


class visualdatalayer():

    def __init__(self, tile, uuid):
        self.tile = tile
        self.uuid = uuid
        self.proj = Proj(tile.cell.proj)

    def as_treelayer(self):

        return f'''<layer-tree-layer checked="Qt::Checked" 
            id="{self.tile.acq_id}_visual_tif_{self.uuid}" 
            expanded="1" name="{self.tile.acq_id}-visual.tif" 
            legend_exp="" providerKey="gdal" 
            source="/vsis3/{self.tile.asset_paths['visual']}">
        <customproperties/>
        </layer-tree-layer>'''

    def as_maplayer(self):
        return f'''
        <maplayer maxScale="0" autoRefreshTime="0" type="raster" hasScaleBasedVisibilityFlag="0" autoRefreshEnabled="0" styleCategories="AllStyleCategories" minScale="1e+08" refreshOnNotifyEnabled="0" refreshOnNotifyMessage="">
      <extent>
        <xmin>{self.tile.cell.bounds[0] - 0.30517578125 * 256} </xmin>
        <ymin>{self.tile.cell.bounds[1] - 0.30517578125 * 256}</ymin>
        <xmax>{self.tile.cell.bounds[2] + 0.30517578125 * 256}</xmax>
        <ymax>{self.tile.cell.bounds[3] + 0.30517578125 * 256 }</ymax>
      </extent>
      <id>{self.tile.acq_id}_visual_tif_{self.uuid}</id>
      <datasource>/vsis3/{self.tile.asset_paths['visual']}</datasource>
      <keywordList>
        <value></value>
      </keywordList>
      <layername>{self.tile.acq_id}-visual.tif</layername>
      <srs>
        <spatialrefsys>
          <wkt>{self.proj.crs.to_wkt()}</wkt>
          <proj4>{self.proj.srs}</proj4>
          <srsid>3094</srsid>
          <srid>{self.proj.crs.to_epsg()}</srid>
          <authid>{self.proj.crs.to_string()}</authid>
          <description>{self.proj.crs.name}</description>
          <projectionacronym>utm</projectionacronym>
          <ellipsoidacronym>WGS84</ellipsoidacronym>
          <geographicflag>false</geographicflag>
        </spatialrefsys>
      </srs>
      <resourceMetadata>
        <identifier></identifier>
        <parentidentifier></parentidentifier>
        <language></language>
        <type></type>
        <title></title>
        <abstract></abstract>
        <links/>
        <fees></fees>
        <encoding></encoding>
        <crs>
          <spatialrefsys>
            <wkt>{self.proj.crs.to_wkt()}</wkt>
            <proj4>{self.proj.srs}</proj4>
            <srsid>3094</srsid>
            <srid>{self.proj.crs.to_epsg()}</srid>
            <authid>{self.proj.crs.to_string()}</authid>
            <description>{self.proj.crs.name}</description>
            <projectionacronym>utm</projectionacronym>
            <ellipsoidacronym>WGS84</ellipsoidacronym>
            <geographicflag>false</geographicflag>
            </spatialrefsys>
        </crs>
        <extent/>
      </resourceMetadata>
      <provider>gdal</provider>
      <noData>
        <noDataList bandNo="1" useSrcNoData="0"/>
        <noDataList bandNo="2" useSrcNoData="0"/>
        <noDataList bandNo="3" useSrcNoData="0"/>
        <noDataList bandNo="4" useSrcNoData="0"/>
      </noData>
      <map-layer-style-manager current="default">
        <map-layer-style name="default"/>
      </map-layer-style-manager>
      <flags>
        <Identifiable>1</Identifiable>
        <Removable>1</Removable>
        <Searchable>1</Searchable>
      </flags>
      <customproperties>
        <property key="identify/format" value="Value"/>
      </customproperties>
      <pipe>
        <rasterrenderer type="multibandcolor" alphaBand="4" opacity="1" redBand="1" blueBand="3" nodataColor="" greenBand="2">
          <rasterTransparency/>
          <minMaxOrigin>
            <limits>MinMax</limits>
            <extent>WholeRaster</extent>
            <statAccuracy>Estimated</statAccuracy>
            <cumulativeCutLower>0.02</cumulativeCutLower>
            <cumulativeCutUpper>0.98</cumulativeCutUpper>
            <stdDevFactor>2</stdDevFactor>
          </minMaxOrigin>
          <redContrastEnhancement>
            <minValue>0</minValue>
            <maxValue>255</maxValue>
            <algorithm>NoEnhancement</algorithm>
          </redContrastEnhancement>
          <greenContrastEnhancement>
            <minValue>0</minValue>
            <maxValue>255</maxValue>
            <algorithm>NoEnhancement</algorithm>
          </greenContrastEnhancement>
          <blueContrastEnhancement>
            <minValue>0</minValue>
            <maxValue>255</maxValue>
            <algorithm>NoEnhancement</algorithm>
          </blueContrastEnhancement>
        </rasterrenderer>
        <brightnesscontrast brightness="0" contrast="0"/>
        <huesaturation colorizeOn="0" colorizeGreen="128" saturation="0" grayscaleMode="0" colorizeRed="255" colorizeStrength="100" colorizeBlue="128"/>
        <rasterresampler maxOversampling="2"/>
      </pipe>
      <blendMode>0</blendMode>
    </maplayer>'''


class clouddatalayer():

    def __init__(self, tile, uuid):
        self.tile = tile
        self.uuid = uuid
        self.proj = Proj(tile.cell.proj)

    def as_treelayer(self):

        return f'''<layer-tree-layer checked="Qt::Checked" 
            id="{self.tile.acq_id}_cloud_mask_{self.tile.cell.zone}_{self.tile.cell.quadkey}_{self.tile.acq_id}_clouds_mask_{self.uuid}" 
            expanded="1" name="{self.tile.acq_id}-cloud-mask {self.tile.cell.zone}/{self.tile.cell.quadkey}/self-clouds-mask" 
            legend_exp="" providerKey="ogr" 
            source="/vsis3/{self.tile.asset_paths['clouds']}|layername={self.tile.cell.zone}/{self.tile.cell.quadkey}/{self.tile.acq_id}-clouds-mask">
        <customproperties/>
        </layer-tree-layer>'''

    def as_maplayer(self):
        return f'''<maplayer refreshOnNotifyEnabled="0" simplifyAlgorithm="0" simplifyMaxScale="1" autoRefreshTime="0" minScale="100000000" wkbType="MultiPolygon" simplifyDrawingHints="1" readOnly="0" maxScale="0" refreshOnNotifyMessage="" styleCategories="AllStyleCategories" simplifyDrawingTol="1" labelsEnabled="0" type="vector" simplifyLocal="1" hasScaleBasedVisibilityFlag="0" autoRefreshEnabled="0" geometry="Polygon">
      <id>{self.tile.acq_id}_cloud_mask_{self.tile.cell.zone}_{self.tile.cell.quadkey}_{self.tile.acq_id}_clouds_mask_{self.uuid}</id>
      <datasource>/vsis3/{self.tile.asset_paths['clouds']}|layername={self.tile.cell.zone}/{self.tile.cell.quadkey}/{self.tile.acq_id}-clouds-mask</datasource>
      <keywordList>
        <value></value>
      </keywordList>
      <layername>{self.tile.acq_id}-cloud-mask 10/{self.tile.cell.quadkey}/{self.tile.acq_id}-clouds-mask</layername>
      <srs>
        <spatialrefsys>
          <wkt>{self.proj.crs.to_wkt()}</wkt>
          <proj4>{self.proj.srs}</proj4>
          <srsid>3094</srsid>
          <srid>{self.proj.crs.to_epsg()}</srid>
          <authid>{self.proj.crs.to_string()}</authid>
          <description>{self.proj.crs.name}</description>
          <projectionacronym>utm</projectionacronym>
          <ellipsoidacronym>WGS84</ellipsoidacronym>
          <geographicflag>false</geographicflag> 
        </spatialrefsys>
      </srs>
      <resourceMetadata>
        <identifier></identifier>
        <parentidentifier></parentidentifier>
        <language></language>
        <type>dataset</type>
        <title></title>
        <abstract></abstract>
        <links/>
        <fees></fees>
        <encoding></encoding>
        <crs>
          <spatialrefsys>
            <wkt></wkt>
            <proj4></proj4>
            <srsid>0</srsid>
            <srid>0</srid>
            <authid></authid>
            <description></description>
            <projectionacronym></projectionacronym>
            <ellipsoidacronym></ellipsoidacronym>
            <geographicflag>false</geographicflag>
          </spatialrefsys>
        </crs>
        <extent/>
      </resourceMetadata>
      <provider encoding="UTF-8">ogr</provider>
      <vectorjoins/>
      <layerDependencies/>
      <dataDependencies/>
      <legend type="default-vector"/>
      <expressionfields/>
      <map-layer-style-manager current="default">
        <map-layer-style name="default"/>
      </map-layer-style-manager>
      <auxiliaryLayer/>
      <flags>
        <Identifiable>1</Identifiable>
        <Removable>1</Removable>
        <Searchable>1</Searchable>
      </flags>
      <renderer-v2 type="singleSymbol" enableorderby="0" forceraster="0" symbollevels="0">
        <symbols>
          <symbol type="fill" alpha="1" clip_to_extent="1" force_rhr="0" name="0">
            <layer locked="0" enabled="1" pass="0" class="SimpleFill">
              <prop k="border_width_map_unit_scale" v="3x:0,0,0,0,0,0"/>
              <prop k="color" v="229,182,54,255"/>
              <prop k="joinstyle" v="bevel"/>
              <prop k="offset" v="0,0"/>
              <prop k="offset_map_unit_scale" v="3x:0,0,0,0,0,0"/>
              <prop k="offset_unit" v="MM"/>
              <prop k="outline_color" v="35,35,35,255"/>
              <prop k="outline_style" v="solid"/>
              <prop k="outline_width" v="0.26"/>
              <prop k="outline_width_unit" v="MM"/>
              <prop k="style" v="solid"/>
              <data_defined_properties>
                <Option type="Map">
                  <Option type="QString" value="" name="name"/>
                  <Option name="properties"/>
                  <Option type="QString" value="collection" name="type"/>
                </Option>
              </data_defined_properties>
            </layer>
          </symbol>
        </symbols>
        <rotation/>
        <sizescale/>
      </renderer-v2>
      <customproperties/>
      <blendMode>0</blendMode>
      <featureBlendMode>0</featureBlendMode>
      <layerOpacity>1</layerOpacity>
      <geometryOptions geometryPrecision="0" removeDuplicateNodes="0">
        <activeChecks type="StringList">
          <Option type="QString" value=""/>
        </activeChecks>
        <checkConfiguration/>
      </geometryOptions>
      <referencedLayers/>
      <referencingLayers/>
      <fieldConfiguration>
        <field name="fid">
          <editWidget type="">
            <config>
              <Option/>
            </config>
          </editWidget>
        </field>
        <field name="cloudCoverPct">
          <editWidget type="">
            <config>
              <Option/>
            </config>
          </editWidget>
        </field>
      </fieldConfiguration>
      <aliases>
        <alias field="fid" index="0" name=""/>
        <alias field="cloudCoverPct" index="1" name=""/>
      </aliases>
      <excludeAttributesWMS/>
      <excludeAttributesWFS/>
      <defaults>
        <default field="fid" expression="" applyOnUpdate="0"/>
        <default field="cloudCoverPct" expression="" applyOnUpdate="0"/>
      </defaults>
      <constraints>
        <constraint field="fid" notnull_strength="1" unique_strength="1" exp_strength="0" constraints="3"/>
        <constraint field="cloudCoverPct" notnull_strength="0" unique_strength="0" exp_strength="0" constraints="0"/>
      </constraints>
      <constraintExpressions>
        <constraint field="fid" desc="" exp=""/>
        <constraint field="cloudCoverPct" desc="" exp=""/>
      </constraintExpressions>
      <expressionfields/>
      <attributeactions>
        <defaultAction key="Canvas" value="{00000000-0000-0000-0000-000000000000}"/>
      </attributeactions>
      <attributetableconfig actionWidgetStyle="dropDown" sortOrder="0" sortExpression="">
        <columns/>
      </attributetableconfig>
      <conditionalstyles>
        <rowstyles/>
        <fieldstyles/>
      </conditionalstyles>
      <storedexpressions/>
      <editform tolerant="1"></editform>
      <editforminit/>
      <editforminitcodesource>0</editforminitcodesource>
      <editforminitfilepath></editforminitfilepath>
      <editforminitcode><![CDATA[]]></editforminitcode>
      <featformsuppress>0</featformsuppress>
      <editorlayout>generatedlayout</editorlayout>
      <editable/>
      <labelOnTop/>
      <widgets/>
      <previewExpression></previewExpression>
      <mapTip></mapTip>
    </maplayer> '''

class treelayer():

    def __init__(self, tile, uuid):
        self.tile = tile
        self.uuid = uuid
        self.visual = visualdatalayer(tile, uuid)
        self.clouds = clouddatalayer(tile, uuid)

    def as_text(self):
        return f'''
            <layer-tree-group checked="Qt::Checked" expanded="1" name="{self.tile.cell.id}">
                <customproperties/>
                {self.clouds.as_treelayer()}
                {self.visual.as_treelayer()}
            </layer-tree-group> '''

class maplayer():

    def __init__(self, tile, uuid):
        self.tile = tile
        self.uuid = uuid
        self.visual = visualdatalayer(tile, uuid)
        self.clouds = clouddatalayer(tile, uuid)
        
    def as_text(self):
        return f'''
                {self.clouds.as_maplayer()}
                {self.visual.as_maplayer()}
             '''


class qlayer():

    def __init__(self, collection):
        self.treelayers = defaultdict(list) 
        self.maplayers = []
        for acq in collection.acquisitions:
            for tile in acq.tiles:
                uuid = uuid4().hex
                self.treelayers[acq].append(treelayer(tile, uuid))
                self.maplayers.append(maplayer(tile, uuid))


    def render_tree(self):
        text = ''
        for acq, layers in self.treelayers.items():
            text += f'<layer-tree-group checked="Qt::Unchecked" expanded="0" name="{acq.acq_id}">'
            text += '<customproperties/>'
            text += '\n'.join(t.as_text() for t in layers)
            text += ' </layer-tree-group>'
        return text

    def render_layers(self):
        return '\n'.join([l.as_text() for l in self.maplayers])

    def as_text(self):
        return f'''<!DOCTYPE qgis-layer-definition>
        <qlr>
           <layer-tree-group checked="Qt::Checked" expanded="1" name="">
            <customproperties/>
            {self.render_tree()}
            </layer-tree-group> 
            <maplayers>
            {self.render_layers()}
            </maplayers>
        </qlr> ''' 